/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 09/24/2002 */

/* The cluster graph implements the graph of cluters of interest
   (e.g. above a threshold). It explicitly stores pointers describing
   the parent-child relationships among the clusters. */

#ifndef CLGRAPH_H
#define CLGRAPH_H

#include "../hash_map/hash_map.h"
#include "../hash_map/growing_hash.h"
#include "../datatypes/cluster.h"
#include "../datatypes/record.h"

struct clgnode; /* A node of the cluster graph */

struct clgnlist{ /* A list of cluster graph nodes */
  clgnode* node;
  clgnlist* next;
};

#pragma pack (4)
struct onedimlinks{ /* For each dimension */
  clgnode* parent; /* We have one parent */
  clgnlist* children; /* And a list of children */
};
typedef struct clgnode{
  uint4 status; /* For all kinds of status information, flags,
		   etc. used by various algorithms. Always leave the
		   high order byte of this word zero. */
  onedimlinks links[1]; /* The actual number of dimensions is set when
			   we initialize the graph. */
  /* These fields are followed by 64 bit counters. The number of
     counters is set when we initialize the graph. */
} clgnode;
#pragma pack()

inline sint8& counteri(clgnode* node, unsigned counteroffset)
{return *((sint8*)((byte*)node+counteroffset));}

inline double& doublecounteri(clgnode* node, unsigned counteroffset)
{return *((double*)((byte*)node+counteroffset));}

class clustergraph{
 public:
  clustergraph(cluster& cl, unsigned numcounters);
  ~clustergraph();
  void read_in_clusters(char* rawclusterfile, unsigned counternum=0);
  /* Reads in a raw file with the clusters of interest - puts their
     traffic in counter number counternum*/
  void find_hvclusters(record& flowID, hash_map& flow_memory,
		       sint8 threshold, unsigned counternum=0);
  /* Populates the cluster graph with the high volume clusters from
     the flow memory keyed by flowID (which must fit cluster c). */
  sint8 totaltraffic(unsigned counter=0){return counter_i(root,counter);}
  sint8& counter_i(clgnode* node, unsigned i)
    {return ((sint8*)((byte*)node+counteroffset))[i];}
  double& doublecounter_i(clgnode* node, unsigned i)
    {return ((double*)((byte*)node+counteroffset))[i];}
  sint8 sumall(clgnlist* list, unsigned cnum);
  /* sums counters stored in list elements at offset */
  growing_hash hvclmem; /* The actual nodes are the values of this 
			   growing_hash */
  cluster* c;
  unsigned numcounters;
  const unsigned counteroffset;
  clgnode* root;
  clgnode** traversalorder;
 private:
  void add_missing_pointers_and_nodes();
  void rebuild_traversal_order();
  clgnlist* addtoorderedlist(clgnlist* l, clgnode* n, unsigned pnum);
  /* using this function we can biuld a list with the elements ordered
     by the value of the pattern number pnum*/
  void sortclusters(int first, int last);
};

clgnlist* addtolist(clgnlist* l, clgnode* n);
void freelist(clgnlist*l);
int listlength(clgnlist*l);

#endif /* CLGRAPH_H */
