/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 07/04/2003 */

#include <stdlib.h>
#include "compression.h"
#include "../readwrite/assume.h"

void decideunidimcompr(clgnode* node);
/* Decides which clusters go into the unidimensional compressed report. */
clgnlist* buildunidimreport(clgnode* node, clgnlist* rest);
/* Builds the report and cleans up after decideunidimcompr. */

static unsigned traffic_counter_offset;
static unsigned estimate_counter_offset;
/* The counter at offset traffic_counter_offset in the cluster graph
   node holds the traffic and the one at estimate_counter_offset is
   used as an auxiliary counter. */
static sint8 threshold;
static unsigned dim;
/* The dimension along which we do the unidimensional compression*/

clgnlist* build_unidim_report(clustergraph* cg, unsigned d, sint8 th){
  if(cg->root==NULL){return NULL;}
  traffic_counter_offset=cg->counteroffset;
  estimate_counter_offset=cg->counteroffset+sizeof(sint8);
  threshold=th;
  dim=d;
  decideunidimcompr(cg->root);
  return buildunidimreport(cg->root,NULL);
}

#define IN_UNIDIM_REPORT 0x2000000
void decideunidimcompr(clgnode* node){
  /* Decides which clusters go into the unidimensional compressed
     report. Uses the second counter. */
  sint8 childrensum=0;
  for(clgnlist* crt=node->links[dim].children;crt!=NULL;crt=crt->next){
    decideunidimcompr(crt->node);
    childrensum+=counteri(crt->node,estimate_counter_offset);
  }
  if(counteri(node,traffic_counter_offset)-childrensum>=threshold){
    counteri(node,estimate_counter_offset)=counteri(node,traffic_counter_offset);
    node->status|=IN_UNIDIM_REPORT;
  } else {
    counteri(node,estimate_counter_offset)=childrensum;    
  }
}

clgnlist* buildunidimreport(clgnlist* nodes, clgnlist* rest){
  if(nodes==NULL){
    return rest;
  }
  rest=buildunidimreport(nodes->next,rest);
  return buildunidimreport(nodes->node,rest);
}

clgnlist* buildunidimreport(clgnode* node, clgnlist* rest){
  rest=buildunidimreport(node->links[dim].children,rest);
  counteri(node,estimate_counter_offset)=0;
  if(node->status&IN_UNIDIM_REPORT){
    node->status&=~IN_UNIDIM_REPORT;
    return addtolist(rest, node);
  } else {
    return rest;
  }
}

static byte ancestor[300];
clgnlist* build_multidim_report(clustergraph* cg, sint8 threshold,
				unsigned add_root){
  clgnlist* result=NULL;
  clgnode** to=cg->traversalorder;
  unsigned traffcoff=cg->counteroffset;
  unsigned estimatecoff=cg->counteroffset+sizeof(sint8);
  unsigned expectedcoff=cg->counteroffset+2*sizeof(sint8);
  unsigned clustersize=cg->c->getbinsize();
  if(cg->hvclmem.size()==0){
    return NULL;
  }
  sint8 totaltraffic=counteri(cg->root,traffcoff);
  assume(clustersize<=300,"Oops, cluster size larger than ancestor (300).\n");
  assume(sizeof(double)==8,"Unexpectedness labels need doubles be 8 bytes.\n");
  for(int i=cg->hvclmem.size()-1;i>=0;i--){
    if(counteri(to[i],traffcoff)<threshold){
      continue;
    }
    counteri(to[i],estimatecoff)=0;
    for(int j=0;j<cg->c->getnumthings();j++){
      sint8 childsum=cg->sumall(to[i]->links[j].children,1);
      if (childsum>counteri(to[i],estimatecoff)){
	counteri(to[i],estimatecoff)=childsum;
      }
    }
    if(counteri(to[i],traffcoff)-counteri(to[i],estimatecoff)>=threshold){
      counteri(to[i],estimatecoff)=counteri(to[i],traffcoff);
      result=addtolist(result,to[i]);
      /* Set the expected traffic of this cluster*/
      double expected=totaltraffic;
      for(int j=0;j<cg->c->getnumthings();j++){
	copy((byte*)(cg->root)-clustersize,ancestor,clustersize);
	copy((byte*)(to[i])-clustersize+cg->c->getoffset(j),
	     ancestor+cg->c->getoffset(j),cg->c->getpattern(j).getbinsize());
	clgnode* ancestornode=((clgnode*)(cg->hvclmem[ancestor]));
	expected=expected*counteri(ancestornode,traffcoff)/totaltraffic;
      }
      doublecounteri(to[i],expectedcoff)=expected;
    }
  }
  for(int i=cg->hvclmem.size()-1;i>=0;i--){
    counteri(to[i],estimatecoff)=0;
  }
  if(add_root && result!=NULL && result->node != cg->root){
    result=addtolist(result,cg->root);
    counteri(cg->root,expectedcoff)=totaltraffic;
  }
  return result;
}
