/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 10/12/2003 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "countingsynopsis.h"
#include "../readwrite/assume.h"
#include "../readwrite/buffer.h"
#include "../readwrite/basicfilereader.h"

static buffer b;

static uint8 last_threshold=1;
static double factor=0;

/* The number of bytes skipped until the next sampled byte. */
static uint8 next_sampled(uint8 threshold){
  double p;
  if(threshold==1){return 0;}
  if(threshold>RAND_MAX>>6){
    p=((double)(rand()))/RAND_MAX+((double)(rand()+1))/RAND_MAX/RAND_MAX;
  } else {
    p=((double)(rand()+1))/RAND_MAX;
  }
  if(threshold!=last_threshold){
    last_threshold=threshold;
    factor=1/log(((double)threshold)/(threshold-1));
  }
  return (uint8)(-factor*log(p));
}

countingsyn::countingsyn(thing* keytyp, unsigned maxs):maxsize(maxs),
  flows(keytyp->getbinsize(),8),keytype(keytyp){
  threshold=1;
}

void countingsyn::addpacket(byte* keyp, uint8 size){
  if(threshold==1){
    (*((uint8*)(flows[keyp])))+=size;
  } else {
    if(flows.has(keyp)){
      (*((uint8*)(flows[keyp])))+=size;
      return;
    }
    if(size>nextsampling){
      (*((uint8*)(flows[keyp])))=size-nextsampling;
      nextsampling=next_sampled(threshold);
    } else {
      nextsampling-=size;
      return;
    }
  }
  while(flows.size()>maxsize){
    uint8 newthreshold=(uint8)ceil(threshold*1.1);
    increasethresholdto(newthreshold);
  }
}

void countingsyn::write_out_syn(char* filename){
  FILE* F;
  if(strcmp(filename,"-")==0){
    F=stdout;
  } else {
    F=fopen(filename,"w");
    assume1(F!=NULL,"Can not open synopsis %s for writing.\n",filename);
  }
  fprintf(F,"threshold %lld\n",threshold);
  for(add_delete_hash_iterator i(flows);!i.at_end();++i){
    b.offset=0;
    keytype->print(b,i.key());
    b.print_char('\0');
    fprintf(F,"%s\t%llu\n",b.buff,*((uint8*)i.value()));
  }
  if(strcmp(filename,"-")!=0){
    fclose(F);
  }
}

void countingsyn::read_in_syn(char* filename){
  basicfilereader f(filename);
  byte* key;
  char* ics="Invalid counting synopsis\n";
  assume(flows.size()==0,
	 "Cannot read in a new synopsis into one that's not empty\n");
  f.succeedordie(f.getnextline(),ics);
  assume(strncmp("threshold ",f.b.buff,10)==0,ics);
  f.b.offset=10;
  f.succeedordie(f.b.parse_unsigned8(&threshold),ics);
  nextsampling=next_sampled(threshold);
  key=(byte*)malloc(keytype->getbinsize());
  assumenotnull(key);
  while(f.getnextline()){
    f.succeedordie(keytype->parse(f.b,key));
    f.succeedordie(f.b.parse_char('\t',ics));
    f.succeedordie(f.b.parse_unsigned8((uint8*)flows[key]));
    assume2(flows.size()<=maxsize,"Synopsis %s larger than max. size %d\n",
	    filename,maxsize);
  }
  free(key);
}

void countingsyn::add_syn(char* filename){
  countingsyn newsyn(keytype,maxsize);
  newsyn.read_in_syn(filename);
  uint8 newthreshold=threshold+newsyn.threshold-1;
  nextsampling=next_sampled(newthreshold);
  for(add_delete_hash_iterator i(flows);!i.at_end();){
    if(newsyn.flows.has(i.key())){
      *(uint8*)i.value()+=*(uint8*)(newsyn.flows[i.key()]);
    } else {
      if((rand()*newthreshold)> RAND_MAX*threshold){
	if(*(uint8*)i.value()>nextsampling){
	  (*((uint8*)i.value()))-=nextsampling;
	  nextsampling=next_sampled(newthreshold);
	} else {
	  nextsampling-=(*((uint8*)i.value()));
	  i.delete_crt_entry(); /* Also moves i to the next entry */
	  continue;
	}
      }
    }
    ++i;
  }
  for(add_delete_hash_iterator i(newsyn.flows);!i.at_end();++i){
    if(flows.has(i.key())){
      continue;
    }
    if((rand()*newthreshold)<=RAND_MAX*newsyn.threshold){
      *(uint8*)flows[i.key()]=*(uint8*)i.value();
    } else {
      if(*(uint8*)i.value()>nextsampling){
	*(uint8*)flows[i.key()]=(*((uint8*)i.value()))-nextsampling;
	nextsampling=next_sampled(newthreshold);
      } else {
	nextsampling-=(*((uint8*)i.value()));
      }
    }
  }
  threshold=newthreshold;
  while(flows.size()>maxsize){
    newthreshold=(uint8)ceil(threshold*1.1);
    increasethresholdto(newthreshold);
  }
}

hash_map_iterator* countingsyn::build_iterator(){
  return flows.build_iterator();
}

void countingsyn::increasethresholdto(uint8 newthreshold){
  nextsampling=next_sampled(newthreshold);
  for(add_delete_hash_iterator i(flows);!i.at_end();){
    if((rand()*newthreshold)> RAND_MAX*threshold){
      if(*(uint8*)i.value()>nextsampling){
	(*((uint8*)i.value()))-=nextsampling;
	nextsampling=next_sampled(newthreshold);
      } else {
	nextsampling-=(*((uint8*)i.value()));
	i.delete_crt_entry(); /* This moves to the next entry */
	continue;
      }
    }
    ++i;
  }
  threshold=newthreshold;
}
