/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 10/12/2003 */

/* A counting synopsis keeps per flow records. It has a fixed maximum
   size. If the number of flows is larger than the maximum size, it
   resorts to sampling as used by the counting samples of Gibbons and
   Matias (hence the name) and the sample and hold of Estan and
   Varghese. Packets (or bytes) are sampled with probability
   1/threshold. If a flow has no sampled packets, it is noe kept in
   the flow memory, otherwise its counter is decremented by the number
   of packets that went by before the first sampled packet. */

#ifndef COUNTINGSYNOPSIS_H
#define COUNTINGSYNOPSIS_H

#include <stdlib.h>
#include "../datatypes/thing.h"
#include "../hash_map/add_delete_hash.h"

class countingsyn{
 public:
  countingsyn(thing* keytype, unsigned maxs);
  void addpacket(byte* keyp, uint8 size=1);
  void write_out_syn(char* filename);
  void read_in_syn(char* filename);
  void add_syn(char* filename);
  hash_map_iterator* build_iterator();
  uint8 get_th(){return threshold;}
 protected:
  void increasethresholdto(uint8 newthreshold);
  const unsigned maxsize;
  thing * const keytype;
  uint8 threshold;
  uint8 nextsampling; /* bytes to skip before the next sampled one */
  add_delete_hash flows;
};

#endif /* COUNTINGSYNOPSIS_H */
