/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 10/13/2003 */

#include <stdlib.h>
#include "flowsynopsis.h"
#include "../readwrite/assume.h"
#include "../readwrite/buffer.h"
#include "../readwrite/basicfilereader.h"

flowsyn::flowsyn(thing* keytyp, unsigned maxs, unsigned seeed):
  keytype(keytyp),maxsize(maxs),seed(seeed),flows(keytyp->getbinsize(),0){
  step=0;
  threshold=0xFFFFFFFF;
  srand(seed);
  selectionhash= new h3(keytype->getbinsize());
}

flowsyn::~flowsyn(){
  delete selectionhash;
}

void flowsyn::addpacket(byte* keyp){
  if(selectionhash->compute(keyp)<=threshold){
    flows.insert(keyp);
  }
  while(flows.size()>maxsize){
    next_step();
  }
}

void flowsyn::write_out_syn(char* filename){
  FILE* F;
  if(strcmp(filename,"-")==0){
    F=stdout;
  } else {
    F=fopen(filename,"w");
    assume1(F!=NULL,"Can not open synopsis %s for writing.\n",filename);
  }
  fprintf(F,"seed %u\n",seed);
  fprintf(F,"step %u\n",step);
  buffer b;
  for(add_delete_hash_iterator i(flows);!i.at_end();++i){
    b.offset=0;
    keytype->print(b,i.key());
    b.print_char('\0');
    fprintf(F,"%s\n",b.buff);
  }
  if(strcmp(filename,"-")!=0){
    fclose(F);
  }
}

void flowsyn::read_in_syn(char* filename){
  basicfilereader f(filename);
  byte* key;
  unsigned s;
  char* ifs="Invalid flow synopsis\n";
  assume(flows.size()==0,
	 "Cannot read in a new synopsis into one that's not empty\n");
  f.succeedordie(f.getnextline(),ifs);
  assume(strncmp("seed ",f.b.buff,5)==0,ifs);
  f.b.offset=5;
  f.succeedordie(f.b.parse_unsigned4(&s),ifs);
  assume2(s=seed,"Seeds for flow synopsis don't match %u %u\n",seed,s);
  f.succeedordie(f.getnextline(),ifs);
  assume(strncmp("step ",f.b.buff,5)==0,ifs);
  f.b.offset=5;
  f.succeedordie(f.b.parse_unsigned4(&step),ifs);
  key=(byte*)malloc(keytype->getbinsize());
  assumenotnull(key);
  while(f.getnextline()){
    f.succeedordie(keytype->parse(f.b,key));
    flows.insert(key);
    assume2(flows.size()<=maxsize,"Synopsis %s larger than max. size %d\n",
	    filename,maxsize);
  }
  free(key);
}

void flowsyn::add_syn(char* filename){
  flowsyn newsyn(keytype,maxsize,seed);
  newsyn.read_in_syn(filename);
  unsigned newstep=(step>newsyn.step?step:newsyn.step);
  while(step<newstep){
    next_step();
  }
  for(add_delete_hash_iterator i(newsyn.flows);!i.at_end();++i){
    flows.insert(i.key());
  }
  while(flows.size()>maxsize){
    next_step();
  }
}

hash_map_iterator* flowsyn::build_iterator(){
  return flows.build_iterator();
}

void flowsyn::next_step(){
  step++;
  threshold=0xFFFFFFFF>>step;
  for(add_delete_hash_iterator i(flows);!i.at_end();){
    if((selectionhash->compute(i.key())<=threshold)){
      ++i;
    } else {
      i.delete_crt_entry();
    }
  }
}
