/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 10/13/2003 */

/* A flow synopsis stores flowIds seen. It has a fixed maximum
   size. If the number of flows is larger than the maximum size, it
   resorts to sampling using a hash function and a threshold: it keeps
   only flowIDs whose hash value is below the threshold. The threshold
   is powers of two and its initial value covers the whole hash
   space. */

#ifndef FLOWSYNOPSIS_H
#define FLOWSYNOPSIS_H

#include <stdlib.h>
#include "../datatypes/thing.h"
#include "../hash_map/add_delete_hash.h"
#include "../hash_map/h3.h"

class flowsyn{
 public:
  flowsyn(thing* keytype, unsigned maxs, unsigned seed);
  ~flowsyn();
  void addpacket(byte* keyp);
  void write_out_syn(char* filename);
  void read_in_syn(char* filename);
  void add_syn(char* filename);
  hash_map_iterator* build_iterator();
  unsigned getsize(){return flows.size();}
  unsigned compensation_factor(){return 1<<step;}
 protected:
  const unsigned maxsize;
  thing* const keytype;
  h3* selectionhash;
  uint32bit threshold;
  unsigned step;
  const unsigned seed;
  add_delete_hash flows;
  void next_step();
};

#endif /* FLOWSYNOPSIS_H */
