/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 06/23/2003 */

/* A one field hierarchy is the hierarchy defined on a single field by
   the patterns associated with it. Algorithms often need to traverse
   parts of the hierarcy (e.g. go through all the patterns whose
   traffic is above a threshold). ofhorder objects encode the order of
   traversal and can produce iterators. The basic traversal is a
   preorder depth first traversal. At each element one can decide to
   prune the search (this skips all descendants of the current
   node). Information about the current node's parent is also
   available. */

#ifndef OFH_H
#define OFH_H

#include <stdlib.h>
#include "../../datatypes/typedefs.h"

#pragma pack(1)
typedef struct ofhelement {
  struct ofhelement* parent;
  struct ofhelement* prune; /* Where to go if we prune the traversal. */
  byte pattern[1]; /* Of course actual patterns can be longer. */
} ofhelement;
#pragma pack()

class ofhorder{
  friend class ofhiterator;
 public:
  ofhorder(unsigned int patternsize):es(sizeof(ofhelement)-1+patternsize){;}
  ~ofhorder(){free(patterns);}
 protected:
  ofhelement* patterns;
  unsigned int numpatterns;
  const unsigned int es; /* Element size. */
  ofhelement* nthpattern(unsigned int n)
    {return (ofhelement*)((byte*)patterns+es*(n));}
};

class ofhiterator{
 public:
  ofhiterator(ofhorder& oo):es(oo.es)
    {crt=first=oo.patterns;done=oo.nthpattern(oo.numpatterns);}
  void reset(){crt=first;}
  const byte* getpattern(){return crt->pattern;}
  const byte* getparentpattern(){return crt->parent->pattern;}
  int havepattern(){return crt!=done;}
  void next(int godeeper=1)
    {if(godeeper){crt=(ofhelement*)((byte*)crt+es);}else{crt=crt->prune;}}
 protected:
  ofhelement* crt;
  ofhelement* first;
  ofhelement* done;
  const unsigned int es; /* Element size. */  
};

#endif /* OFH_H */
