#!/usr/local/bin/perl -w

#feed netflow to autofocus, taking care to feed files in order or something

use strict;

my $debug;

$debug and print STDERR "HERE WE GO (STDERR)!\n";
system("date >>log/awake.log");

if (-e '/data5/autofocus.reports/target/LOCK')
{
	die "still locked!";
	exit 1;
}
else
{
	system("echo $$ > /data5/autofocus.reports/target/LOCK");
	die "echo!" if $?;
	my $temp = `cat /data5/autofocus.reports/target/LOCK`;
	die "cat!" if $?;
	chomp $temp;
	if ($temp ne $$)
	{
		die "locking problem!";
	}
}

my $files_db = 'files.db';
my @processed_files = `cat $files_db`;
my $timestamp = shift @processed_files;
die "files_db!" if $?;

my $flow_merge_exe = '/usr/local/netflow/bin/flow-merge -m ';
my $flow_export_exe = '/usr/local/netflow/bin/flow-export';
chomp @processed_files;

@processed_files = sort {$b cmp $a} @processed_files;

my @dirs = ('/data3/netflow.inr-000', '/data3/netflow.inr-666',
            '/data3/netflow.inr-201','/data3/netflow.inr-202',
            '/data3/netflow.inr-001','/data3/netflow.inr-002');

my $dirs_str = join "/* ", @dirs;
$dirs_str.="/*";

$debug and print "dirs_str: $dirs_str\n";
my @current_files;
foreach my $dir (@dirs)
{
	push @current_files, `find -L $dir -name 'ft*'`;
}

die "ls!" if $?;

chomp @current_files;

$debug and print "scalar current_files ".(scalar @current_files)."\n";
@current_files = grep { /^\/data.*ft/ } @current_files;
$debug and print "scalar current_files after grep ".(scalar @current_files)."\n";
@current_files = sort {$b cmp $a} @current_files;
$debug and print "scalar current_files after grep+sort ".(scalar @current_files)."\n";

my @candidate_files;

OUTER: foreach my $current_file (@current_files)
{
	foreach my $processed_file (@processed_files)
	{
#		$debug and print "#$current_file# eq #$processed_file#\n";
		next OUTER if ($current_file eq $processed_file);
	}
	push @candidate_files, $current_file;
}

if (scalar @candidate_files == 0)
{
	system("rm /data5/autofocus.reports/target/LOCK");
	die "no new candidates!";
}

$debug and print "Candidates:\n".(join "\n", @candidate_files)."\n";

my @departed_files;
OUTER2: foreach my $processed_file (@processed_files)
{
	foreach my $current_file (@current_files)
	{
#		$debug and print "$current_file eq $processed_file\n";
		next OUTER2 if ($current_file eq $processed_file);
	}
	push @departed_files, $processed_file;
}

$debug and print "Departed:\n".(join "\n", @departed_files)."\n";

@candidate_files = sort by_time (@candidate_files);

$debug and print "SORTED candidate_files:\n".(join "\n", @candidate_files)."\n";


#must not be earlier than previous latest
my @qualified_candidate_files = grep {file_to_time($_) > $timestamp} 
                                @candidate_files;

$debug and print "sufficently new qualified_candidate_files:\n".
                 (join "\n", @qualified_candidate_files)."\n";

my $current_time = `date +%s`;
chomp $current_time;

#also must be an hour old
@qualified_candidate_files = grep {file_to_time($_) < $current_time - 3590} 
                                @qualified_candidate_files;

$debug and print "sufficently but not too new qualified_candidate_files:\n".
                 (join "\n", @qualified_candidate_files)."\n";

if (scalar @qualified_candidate_files == 0)
{
	system("rm /data5/autofocus.reports/target/LOCK");
	die "no qualified candidates!";
}

#don't choke on too many...make this number a multiple of # of routers - 1
$#qualified_candidate_files = 23 if ($#qualified_candidate_files > 23); 

$debug and print "after truncating, qualified_candidate_files:\n".
                 (join "\n", @qualified_candidate_files)."\n";

my $candidate_files_str = join " ", @qualified_candidate_files;

$debug and print "$flow_merge_exe $candidate_files_str| $flow_export_exe -f 2 | ./process_data.pl\n";
system("$flow_merge_exe $candidate_files_str| $flow_export_exe -f 2 2>>log/process_data.err  | ./process_data.pl 2>log/process_data.err");
die "flow-export/autofocus problem!" if $?;

my $last_file = $qualified_candidate_files[-1];
my $last_file_time = file_to_time($last_file);

$debug and print "$last_file has time $last_file_time (last)\n";
my @new_processed_files = @qualified_candidate_files;
foreach my $processed_file (@processed_files)
{
	unless (grep { /\Q$processed_file\E/ } @departed_files)
	{
		push @new_processed_files, $processed_file;
	}
}

@new_processed_files = sort {$b cmp $a} @new_processed_files;

#$debug and print "new_processed_files\n".(join "\n", @new_processed_files)."\n";
open FILESDB, ">$files_db" or die "could not open $files_db for over-write!";
print FILESDB "$last_file_time\n";
foreach my $new_processed_file (@new_processed_files)
{
	print FILESDB "$new_processed_file\n";
}
close FILESDB;

$debug and print "all done!\n";

system("rm /data5/autofocus.reports/target/LOCK");

sub by_time
{
	#ft-v05.2004-04-21.141501-0700
	my $str1;
	my $str2;
	my $time = $a;
	$time =~ s/^.*ft-v05\.(.*)/$1/;
	my ($y, $m, $d, $mess, $g) = split /[-.]/, $time;
	my $H = substr ($mess, 0, 2);
	my $M = substr ($mess, 2, 2);
	my $S = substr ($mess, 4, 2);
	$g =~ s/^0//;
	$g =~ s/0*$//;
	my $epoch_time_cmd = "date -v-$g"."H -v$y"."y -v$m"."m -v$d"."d -v$H".
	"H -v$M"."M -v$S"."S \"+%s\"";
	#print                "date -v-$g"."H -v$y"."y -v$m"."m -v$d"."d -v$H".
	#"H -v$M"."M -v$S"."S \"+%s\"";
	my $epoch_time = `$epoch_time_cmd`;
	chomp $epoch_time;
	$str1 = $epoch_time;
	$time = $b;
	$time =~ s/^.*ft-v05\.(.*)/$1/;
	($y, $m, $d, $mess, $g) = split /[-.]/, $time;
	$H = substr ($mess, 0, 2);
	$M = substr ($mess, 2, 2);
	$S = substr ($mess, 4, 2);
	$g =~ s/^0//;
	$g =~ s/0*$//;
	$epoch_time_cmd = "date -v-$g"."H -v$y"."y -v$m"."m -v$d"."d -v$H".
	"H -v$M"."M -v$S"."S \"+%s\"";
	#print                "date -v-$g"."H -v$y"."y -v$m"."m -v$d"."d -v$H".
	#"H -v$M"."M -v$S"."S \"+%s\"";
	$epoch_time = `$epoch_time_cmd`;
	chomp $epoch_time;
	$str2 = $epoch_time;
	return $str1 <=> $str2;
}

sub file_to_time
{
	my $time = shift;
	$time =~ s/^.*ft-v05\.(.*)/$1/;
	my ($y, $m, $d, $mess, $g) = split /[-.]/, $time;
	my $H = substr ($mess, 0, 2);
	my $M = substr ($mess, 2, 2);
	my $S = substr ($mess, 4, 2);
	$g =~ s/^0//;
	$g =~ s/0*$//;
	my $epoch_time_cmd = "date -v-$g"."H -v$y"."y -v$m"."m -v$d"."d -v$H".
	"H -v$M"."M -v$S"."S \"+%s\"";
	#print                "date -v-$g"."H -v$y"."y -v$m"."m -v$d"."d -v$H".
	#"H -v$M"."M -v$S"."S \"+%s\"";
	my $epoch_time = `$epoch_time_cmd`;
	chomp $epoch_time;
	return $epoch_time;
}
