/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 07/18/2002 */

#include <string.h>
#include <assert.h>
#include "prefix.h"
#include "ip.h"
#include "ipofm.h"
#include "../../readwrite/assume.h"

void prefix::print(buffer& b, const byte* data){
  if(ipofprefix(data)==0 && maskofprefix(data)==0){
    b.print_char('*');
    return;
  }
  base_field->print(b,(byte*)(&ipofprefix(data)));
  b.offset+=sprintf(b.buff+b.offset,"/%u",
		     (unsigned)maskofprefix(data));
}

int prefix::parse(buffer& b, byte* data){
  if(b.buff[b.offset]=='*'){
    maskofprefix(data)=0;
    ipofprefix(data)=0;
    b.offset++;
    return 1;
  }
  if(!base_field->parse(b,(byte*)(&ipofprefix(data)))){
    return 0;
  }
  if(!b.parse_char('/',"\"/\" missing from prefix.")){
    return 0;
  }
  if(!b.parse_unsigned1(&maskofprefix(data))){
    b.errorstring="Cannot parse mask length.";
    return 0;
  }
  if(maskofprefix(data)>32){
    b.errorstring="Invalid mask length (must be between 0 and 32).";
    return 0;
  }
  return 1;
}

int prefix::match(const byte* pdata, const byte* fdata){
  return prefixoflengthx(*((uint4*)fdata),maskofprefix(pdata)) ==
    ipofprefix(pdata);
}

int prefix::isstar(const byte* data){
  return maskofprefix(data)==0 && ipofprefix(data)==0;
}

void prefix::setwildcard(byte* data){
  maskofprefix(data)=0;
  ipofprefix(data)=0;
}

void prefix::settoparent(const byte* child, byte* parent){
  if(maskofprefix(child)>8){
    maskofprefix(parent)=maskofprefix(child)-1;
    ipofprefix(parent)=prefixoflengthx(ipofprefix(child),maskofprefix(parent));
  } else {
    maskofprefix(parent)=0;
    ipofprefix(parent)=0;
  }
}

int prefix::compare(const byte* data1, const byte* data2){
  assert(prefixoflengthx(ipofprefix(data1),maskofprefix(data1)) == ipofprefix(data1));
  assert(prefixoflengthx(ipofprefix(data2),maskofprefix(data2)) == ipofprefix(data2));
  if(maskofprefix(data1) == maskofprefix(data2)){
    // same mask length
    if(ipofprefix(data1) == ipofprefix(data2)){
      return P_EQUAL;
    } else {
      return P_UNRELATED;
    }
  }
  if(maskofprefix(data1) > maskofprefix(data2)){
    // second might include first
    if(prefixoflengthx(ipofprefix(data1),maskofprefix(data2)) == ipofprefix(data2)){
      return P_SECOND_INCLUDES_FIRST;
    } else {
      return P_UNRELATED;
    }
  }
  // first might include second
  if(prefixoflengthx(ipofprefix(data2),maskofprefix(data1)) == ipofprefix(data1)){
    return P_FIRST_INCLUDES_SECOND;
  } else {
    return P_UNRELATED;
  }
}

int prefix::getdepth(const byte* data){
  if(maskofprefix(data)==0){
    return 0;
  } else {
    return maskofprefix(data)-7;
  }
}

int prefix::comesfirst(const byte* p1, const byte* p2){
  if(ipofprefix(p1)<ipofprefix(p2)){
    return 1;
  }
  if(ipofprefix(p1)==ipofprefix(p2) && maskofprefix(p1)<maskofprefix(p2)){
    return 1;
  }
  return 0;
}

ofm* prefix::buildofm(){
  ipofm* result =new ipofm();
  assumenotnull(result);
  return result;
}
