/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 07/18/2002 */

/* This header describes the IP prefix pattern. */

#ifndef PREFIX_H
#define PREFIX_H

#include "../pattern.h"

/* The internal binary representation of a prefix. */
#pragma pack(1)
struct prefixstruct{
  uint4 ipaddr;
  uint1 masklen;
};
#pragma pack()

inline uint4& ipofprefix(const void* data) {
  return ((prefixstruct*)data)->ipaddr;}
inline uint1& maskofprefix(const void* data) {
  return ((prefixstruct*)data)->masklen;}

class prefix:public pattern{
 public:
  prefix(class ip& f){base_field=&f;}
  unsigned int getbinsize(){return sizeof(prefixstruct);}
  unsigned int getmaxhrsize(){return 18;}
  void print(buffer& b, const byte* data);
  int parse(buffer& b, byte* data);
  int match(const byte* pdata, const byte* fdata);
  /* Does prefix pdata match the ip address fdata? */
  int isstar(const byte* data);
  void setwildcard(byte* data); 
  /* Sets the pattern to "don't care" */
  void settoparent(const byte* child, byte* parent);
  field& basefield(){return *(field*)base_field;}
  int compare(const byte* data1, const byte* data2);
  int gethdepth(){return 26;}
  int getdepth(const byte* data);
  int comesfirst(const byte* p1, const byte* p2);
  class ofm* buildofm();
 private:
  class ip* base_field;
};

/* A couple of helper functions here */
inline uint4 mostsignificantbitx(uint4 ip, int x){
  /* 0 for the MSB, 31 for the LSB */
  return (ip<<x)>>31;
}
inline uint4 prefixoflengthx(uint4 ip, int x){
  /* x from 0 to 32 inclusive */
  if(x==0){return 0;}
  return (ip>>(32-x))<<(32-x);
}
inline uint4 setmsbitx(uint4 ip, int x){
  return ip|(0x80000000>>x);
}
inline uint4 clearmsbitx(uint4 ip, int x){
  return ip&(~(0x80000000>>x));
}

#endif /* PREFIX_H */
