/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 07/29/2002 */

/* A pattern represents a set of field values and patterns define a
   hierarchy on the field. For example an IP prefix is a pattern that
   represents a set of IP addresses, a port range represents a set of
   port numbers. */

#ifndef PATTERN_H
#define PATTERN_H

#include "atom.h"
#include "field.h"

class pattern: public atom{
 public:
  virtual int match(const byte* pdata, const byte* fdata)=0;
  /* Does the pattern stored at pdata match the value at fdata? */
  virtual int isstar(const byte* data)=0;
  /* Is the pattern "don't care" (match all) pattern? */
  virtual void setwildcard(byte* data)=0; 
  /* Sets the pattern to "don't care" */
  virtual void settoparent(const byte* child, byte* parent)=0;
  /* Sets 'parent' to the parentd 'child' */
  virtual field& basefield()=0;
  /* The binary size of the base type we can match against */
  virtual int compare(const byte* data1, const byte* data2)=0;
  /* Determines the relation between two patterns of the same type */
  virtual int gethdepth()=0; /* The depth of the hierarchy */
  virtual int getdepth(const byte* data)=0; /* Depth in the hierarchy */
  virtual int comesfirst(const byte* p1, const byte* p2)=0;
  /* Returns true if p1 comes before p2 in the (depth first) preorder
     traversal (false if they are equal) */
  void printname(buffer& b){basefield().printname(b);};
  virtual class ofm* buildofm()=0;
  /* ofm objects enable algorithms on the hierarchies defined by patterns */
};

/* The valid returns values for compare */
#define P_EQUAL                  0
#define P_FIRST_INCLUDES_SECOND  1
#define P_SECOND_INCLUDES_FIRST  2
#define P_UNRELATED              3

#endif /* PATTERN_H */
