/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 07/30/2002 */

#include <stdlib.h>
#include <string.h>
#include "portofm.h"
#include "port.h"
#include "prange.h"
#include "../../readwrite/assume.h"

portofm::portofm(){
  memset(counters,'\0',8*65537);
}

void portofm::newpacket(const byte* fvalue, uint8 increment){
  if(isdefport(fvalue)){
    counters[portnum(fvalue)]+=increment;
  } else {
    counters[65536]+=increment;
  }
}

ofhorder* portofm::buildorderoflargepatterns(uint8 threshold){
  portofmorder* result =new portofmorder(counters,threshold);
  assumenotnull(result);
  return result;
}

portofmorder::portofmorder(uint8* counters, uint8 threshold):ofhorder(4){
  uint8 lowtraffic=0;
  uint8 hightraffic=0;
  numpatterns=0;
  for(int i=0;i<1024;i++){
    lowtraffic+=counters[i];
    if(counters[i]>=threshold){
      numpatterns++;
    }
  }
  if(lowtraffic>=threshold){
    numpatterns++;
  }
  for(int i=1024;i<65536;i++){
    hightraffic+=counters[i];
    if(counters[i]>=threshold){
      numpatterns++;
    }
  }
  if(hightraffic>=threshold){
    numpatterns++;
  }
  uint8 total=lowtraffic+hightraffic+counters[65536];
  if(total>=threshold){
    numpatterns++;
  }
  if(numpatterns==0){
    patterns=NULL;
    return;
  }
  patterns=(ofhelement*)malloc(es*numpatterns);
  assumenotnull(patterns);
  ofhelement* done=nthpattern(numpatterns);
  patterns[0].parent=patterns;
  patterns[0].prune=done;
  minpnum(patterns[0].pattern)=0;
  maxpnum(patterns[0].pattern)=65535;
  ofhelement* crt=nthpattern(1);

  if(lowtraffic>=threshold){
    crt->parent=patterns;
    ofhelement* crtparent=crt;
    minpnum(crt->pattern)=0;
    maxpnum(crt->pattern)=1023;
    crt=(ofhelement*)((byte*)crt+es);
    for(int i=0;i<1024;i++){
      if(counters[i]>=threshold){
	crt->parent=crtparent;
	crt->prune=(ofhelement*)((byte*)crt+es);
	minpnum(crt->pattern)=maxpnum(crt->pattern)=i;
	crt=(ofhelement*)((byte*)crt+es);
      }
    }
    /* We still need to set the prune pointer of the lowports element */
    nthpattern(1)->prune=crt;
  }

  if(hightraffic>=threshold){
    crt->parent=patterns;
    crt->prune=done;
    minpnum(crt->pattern)=1024;
    maxpnum(crt->pattern)=65535;
    ofhelement* crtparent=crt;
    crt=(ofhelement*)((byte*)crt+es);
    for(int i=1024;i<65536;i++){
      if(counters[i]>=threshold){
	crt->parent=crtparent;
	crt->prune=(ofhelement*)((byte*)crt+es);
	minpnum(crt->pattern)=maxpnum(crt->pattern)=i;
	crt=(ofhelement*)((byte*)crt+es);
      }
    }
  }
}

