/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 07/29/2002 */

#include "prange.h"
#include "port.h"
#include "portofm.h"
#include "../../readwrite/assume.h"

void prange::print(buffer& b, const byte* data){
  if(minpnum(data)==0 && maxpnum(data)==65535){
    b.print_char('*');
    return;
  }
  if(minpnum(data)==maxpnum(data)){
    b.print_unsigned2(minpnum(data));
  } else {
    b.offset+=sprintf(b.buff+b.offset,"%u-%u",minpnum(data),maxpnum(data));
  }
}

int prange::parse(buffer& b, byte* data){
  if(b.parse_char('*')){
    minpnum(data)=0;
    maxpnum(data)=65535;
    return 1;
  }
  if(!b.parse_unsigned2(&minpnum(data))){
    return 0;
  }
  if(!b.parse_char('-')){
    maxpnum(data)=minpnum(data);
    return 1;
  }
  if(!b.parse_unsigned2(&maxpnum(data))){
    return 0;
  }
  if(minpnum(data)>maxpnum(data)){
    b.errorstring="Port range limits in wrong order.";
    return 0;
  }
  return 1;
}

int prange::match(const byte* pdata, const byte* fdata){
  if(isdefport(fdata)){
    return minpnum(pdata)<=portnum(fdata) && maxpnum(pdata)>=portnum(fdata);
  } else {
    return minpnum(pdata)==0 && maxpnum(pdata)==65535;
  }
}

int prange::isstar(const byte* data){
  return minpnum(data)==0 && maxpnum(data)==65535;
}

void prange::setwildcard(byte* data){
  minpnum(data)=0;
  maxpnum(data)=65535;
}

void prange::settoparent(const byte* child, byte* parent){
  if(minpnum(child)==maxpnum(child)){
    if(minpnum(child)<1024){
      minpnum(parent)=0;
      maxpnum(parent)=1023;
    } else {
      minpnum(parent)=1024;
      maxpnum(parent)=65535;
    }
  } else {
    minpnum(parent)=0;
    maxpnum(parent)=65535;
  }
}

int prange::compare(const byte* data1, const byte* data2){
  if(minpnum(data1)<=minpnum(data2) && maxpnum(data1)>=maxpnum(data2)){
    if(minpnum(data1)==minpnum(data2) && maxpnum(data1)==maxpnum(data2)){
      return P_EQUAL;
    } else {
      return P_FIRST_INCLUDES_SECOND;
    }
  }
  if(minpnum(data2)<=minpnum(data1) && maxpnum(data2)>=maxpnum(data1)){
    return P_SECOND_INCLUDES_FIRST;
  } else {
    return P_UNRELATED;
  }  
}

int prange::getdepth(const byte* data){
  if(minpnum(data)==maxpnum(data)){
    return 2;
  }
  if(minpnum(data)==0 && maxpnum(data)==65535){
    return 0;
  } else {
    return 1;
  }
}

int prange::comesfirst(const byte* p1, const byte* p2){
  if(minpnum(p1)<minpnum(p2)){
    return 1;
  }
  if(minpnum(p1)==minpnum(p2) && maxpnum(p1)>maxpnum(p2)){
    return 1;
  }
  return 0;
}

ofm* prange::buildofm(){
  portofm* result =new portofm();
  assumenotnull(result);
  return result;
}
