/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 06/21/2003 */

#include <stdlib.h>
#include <string.h>
#include "growing_hash.h"
#include "../readwrite/assume.h"

static unsigned int sizes[47]={41, 53, 79, 107, 157, 223, 307, 431, 613,
			       863, 1217, 1723, 2437, 3449, 4871, 6899,
			       9743, 13781, 19483, 27581, 38971, 55109,
			       77951, 110221, 155887, 220447, 311743,
			       440893, 623521, 881743, 1246997, 1763491,
			       2493949, 3526987, 4987901, 7053971,
			       9975803, 14107921, 19951597, 28215809,
			       39903197, 56431657, 79806341, 112863217,
			       159612679, 225726419, 319225391};

/* This array stores the possible sizes for the hash table. The first
   number is the first prime larger than 2^5.25, the second is the
   first prime larger than 2^5.75, the third is larger than 2^6.25 and
   so on until the last one which is above 2^28.25 */

growing_hash::growing_hash(unsigned int keysize, unsigned int valuesize):
  hash_map(keysize,valuesize){
  ts=sizes[0];
  table=(entrylist**)malloc(sizeof(entrylist*)*ts);
  assumenotnull(table);
  memset(table,'\0',sizeof(entrylist*)*ts);
  blocks[0]=(entrylist*)malloc(es*sizes[0]);
  assumenotnull(blocks[0]);
  memset(blocks[0],'\0',es*sizes[0]);
  numblocks=1;
  entriesinlastblock=sizes[0];
  entriesusedinlastblock=0;
}

byte* growing_hash::add_key(const byte* key, entrylist** crtp){
  if(entriesusedinlastblock==entriesinlastblock){
    /* Need to extend the growing_hash. */
    assume(numblocks<47,"Too many entries for growing_hash\n");
    free(table);
    ts=sizes[numblocks];
    table=(entrylist**)malloc(sizeof(entrylist*)*ts);
    assumenotnull(table);
    memset(table,'\0',sizeof(entrylist*)*ts);
    entriesinlastblock=sizes[numblocks]-sizes[numblocks-1];
    entriesusedinlastblock=0;
    blocks[numblocks]=(entrylist*)malloc(es*entriesinlastblock);
    assumenotnull(blocks[numblocks]);
    memset(blocks[numblocks],'\0',es*entriesinlastblock);
    for (int i=0;i<numblocks;i++){
      int numentries=(i==0?sizes[0]:sizes[i]-sizes[i-1]);
      entrylist* e;
      unsigned int index;
      for(int j=0;j<numentries*es;j+=es){
	e=(entrylist*)((byte*)(blocks[i])+j);
	index=hashfun(e->entry)%ts;
	e->next=table[index];
	table[index]=e;
      }
    }
    numblocks++;
    for(crtp=table+hashfun(key)%ts;*crtp!=NULL;crtp=&((*crtp)->next)){
      assume(!equals(key,(*crtp)->entry,ks),"Duplicate detected in growing_hash\n");
    }
  }
  *crtp=(entrylist*)((byte*)(blocks[numblocks-1])+es*entriesusedinlastblock);
  copy(key,(*crtp)->entry,ks);
  entriesusedinlastblock++;
  return (*crtp)->entry+ks;
}


unsigned int growing_hash::size(){
  if(numblocks==1){
    return entriesusedinlastblock;
  }
  return entriesusedinlastblock+sizes[numblocks-2];
}


hash_map_iterator* growing_hash::build_iterator(){
  return new growing_hash_iterator(*this);
}

growing_hash_iterator::growing_hash_iterator(growing_hash& map):
  hash_map_iterator(map){
  m=&map;
  if(map.numblocks==1 && map.entriesusedinlastblock==0){
    crtentry=NULL;
    return;
  }
  crtblock=0;
  firstentry=crtentry=map.blocks[0];
  if(map.numblocks>1){
    lastentry=(entrylist*)((byte*)crtentry+es*(sizes[0]-1));
  } else {
    lastentry=(entrylist*)((byte*)crtentry+es*(map.entriesusedinlastblock-1));
  }
}

int growing_hash_iterator::at_end(){
  return crtentry==NULL;
}

void growing_hash_iterator::operator++(){
  if(crtentry<lastentry){
    crtentry=(entrylist*)((byte*)crtentry+es);
  } else {
    getmoreentries();
  }
}

void growing_hash_iterator::getmoreentries(){
  /* First check whether we have any entries left in crtblock */
  unsigned int scb=(crtblock==0?sizes[0]:sizes[crtblock]-sizes[crtblock-1]);
  unsigned int seen=((byte*)lastentry-(byte*)(m->blocks[crtblock]))/es+1;
  if(seen<scb){
    /* Need to see whether the unseen entries are actually used */
    if(m->numblocks==crtblock+1){
      if(m->entriesusedinlastblock==seen){
	crtentry=NULL;
	return;
      } else {
	lastentry=(entrylist*)((byte*)(m->blocks[crtblock])+
			       es*(m->entriesusedinlastblock-1));
      }
    } else {
      lastentry=(entrylist*)((byte*)(m->blocks[crtblock])+es*(scb-1));
    }
    crtentry=(entrylist*)((byte*)crtentry+es);
    return;
  }
  if(crtblock==m->numblocks-1){
    crtentry=NULL;
    return;
  }
  crtblock++;
  if(m->numblocks>crtblock+1){
    crtentry=m->blocks[crtblock];
    lastentry=(entrylist*)((byte*)crtentry+
			   es*(sizes[crtblock]-sizes[crtblock-1]-1));
    return;
  }
  /* We are at the last block */
  if(m->entriesusedinlastblock==0){
    crtentry==NULL;
  } else {
    crtentry=m->blocks[crtblock];
    lastentry=(entrylist*)((byte*)crtentry+es*(m->entriesusedinlastblock-1));
  }
}



