/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 06/20/2003 */

/* Our growing_hash is a hash_map that can only grow (you can add
   entries, but not delete. Overhead is 2 pointers per entry and
   unless there are very few entries 0% to 41% of the allocated memory
   is unused enteries. The iterator traverses the entries in the order
   they were created. */

#ifndef GROWINGHASH_H
#define GROWINGHASH_H

#include <stdlib.h>
#include "hash_map.h"

class growing_hash:public hash_map{
  friend class growing_hash_iterator;
 public:
  growing_hash(unsigned int keysize, unsigned int valuesize);
  /* If the key has no entry associated, it creates one */
  unsigned int size();
  hash_map_iterator* build_iterator();
 protected:
  unsigned int entriesinlastblock;
  unsigned int entriesusedinlastblock;
  byte* add_key(const byte* key,entrylist** crtp);
};

class growing_hash_iterator:public hash_map_iterator{
 public:
  growing_hash_iterator(growing_hash& map);
  int at_end();
  void operator++();
  int at_first(){return crtentry==firstentry;}
  void moveto(int position); /* Equivalent to calling ++ 'position'
				times right after the iterator was
				created. */
 protected:
  void getmoreentries();
  struct entrylist* firstentry;
  struct entrylist* lastentry;
  unsigned int crtblock;
  growing_hash* m;
};


#endif /* GROWINGHASH_H */
