/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 07/25/2003 */

/* This program computes the number of bytes, packets and flows in the
   timestampless trace it gets on stdin. */

#include "../datatypes/ip/ip.h"
#include "../datatypes/port/port.h"
#include "../datatypes/protocol/proto.h"
#include "../datatypes/compound.h"
#include "../datatypes/counter64.h"
#include "../readwrite/asciifile.h"
#include "../algorithms/flowsynopsis.h"

ip srcip("Source IP");
ip destip("Dest. IP");
port srcport("SrcP");
port destport("DestP");
proto protocol("Pro");
compound flowID(&srcip,&destip,&protocol,&srcport,&destport);
counter64 size("Size");
compound flowID_and_size(&srcip,&destip,&protocol,&srcport,&destport,&size);


int main(int argc,char* argv[]){
  uint8 numpackets=0;
  uint8 numbytes=0;
  int idlen=flowID.getbinsize();
  flowsyn flowtable(&flowID,10000000,283);
  asciifilereader trace("-",flowID_and_size);
  byte* headerdata;
  while(headerdata=trace.getnextrecord()){
    numpackets++;
    numbytes+=*(uint8*)(headerdata+idlen);
    flowtable.addpacket(headerdata);
  }
  printf("%llu bytes %llu packets %u flows\n",numbytes,numpackets,
	 flowtable.getsize()*flowtable.compensation_factor());
}
