/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 08/05/2002 */

/* The humanizer transforms an ascii file into something easier to
   read for humans: large numbers are converted to Kilobytes(packets,
   whatever) or Megabytes, etc.; 0-1023 is written as lowports,
   1024-65536 as highports and it takes care not to convert port
   numbers above 999 into "kiloportnumbers". Everything else is left
   unaltered except that it tries to preserve indentation by adding
   spaces when converting port ranges. Takes input on stdin and writes
   output to stdout. */

#include "../readwrite/buffer.h"
#include "../readwrite/basicfilereader.h"

buffer word;
buffer output;
unsigned wordnumber;
unsigned outputoffsetatstartofword;

basicfilereader f("-");
// Reads the next word into the word buffer. A word starts with a
// nonspace followed by a variable number of nonspace characters
// followed by all the space characters that come in the input line
// before the next word or the newline. When we go to the next line
// for the next word, this function also prints the output buffer and
// reinitializes it.
int getnextword(){
  if(f.getlinenum()!=0 && word.buff[word.offset]==' '){
    // Handle the padding of the current word
    while(output.offset-outputoffsetatstartofword<word.offset){
      output.print_char(' ');
    }
    // Copy over the extra spaces
    while(word.buff[word.offset]!='\0'){
      if(output.offset-outputoffsetatstartofword<=word.offset){
	output.print_char(word.buff[word.offset]);
      }
      word.offset++;
    }
    if(output.buff[output.offset-1]!=' '&&output.buff[output.offset-1]!='\t'){
      output.print_char(' ');
    }
  }
  if(f.b.buff[f.b.offset]=='\0' || f.getlinenum()==0){
    // We go on to the next line
    if(f.getlinenum()!=0){
      output.buff[output.offset]='\0';
      printf("%s\n",output.buff);
    }
    output.offset=0;
    wordnumber=0;
    if(f.getnextline(0)){// Next line, don't skip comments
      // Copy over the leading spaces
      if(f.b.offset>0){
	memcpy(output.buff,f.b.buff,f.b.offset);
	output.offset=f.b.offset;
      }
    } else {
      return 0;
    }
  }
  int oldoffset=f.b.offset;
  while(f.b.buff[f.b.offset]!=' ' && f.b.buff[f.b.offset]!='\t' &&
	f.b.buff[f.b.offset]!='\0'){
    f.b.offset++;
  }
  while(f.b.buff[f.b.offset]==' ' || f.b.buff[f.b.offset]=='\t'){
    f.b.offset++;
  }
  memcpy(word.buff,f.b.buff+oldoffset,f.b.offset-oldoffset);
  word.buff[f.b.offset-oldoffset]='\0';
  wordnumber++;
  word.offset=0;
  outputoffsetatstartofword=output.offset;
  return 1;
}

void processunstructuredword(){
// This just takes the word, assumes that all numbers not followed by
// a '.'  represent bytes of traffic, and converts them into human
// readable form (with Ks, Ms, Gss) while leaving everything else
// unchanged.
  uint8 value;
  while(word.buff[word.offset]!='\0'){
    if(isdigit(word.buff[word.offset])){
      if(!word.parse_unsigned8(&value)){
	f.b.errorstring=word.errorstring;
	f.printbuffererror();
      }
      if(word.buff[word.offset]=='-' && isdigit(word.buff[word.offset+1]) &&
	 (value==0 || value==1024)){
	// This might be a port range
	unsigned numendoffset=word.offset;
	uint8 v2;
	word.offset++;
	if(word.parse_unsigned8(&v2)&&((value==0 && v2==1023)|| 
				       (value==0 && v2==65535)||
				       (value==1024 && v2==65535))){
	  if(value==0 && v2==1023){
	    output.print_string("lowports");
	    if(word.buff[word.offset]==' '){
	      output.print_string("  ");
	    }
	  }
	  if(value==0 && v2==65535){
	    output.print_string("*");
	  }
	  if(value==1024 && v2==65535){
	    output.print_string("highports");
	    if(word.buff[word.offset]==' '){
	      output.print_string(" ");
	    }
	  }
	  continue;
	} else {
	  // This is not a port range
	  word.offset=numendoffset; 
	}
      }
      if((value>65535 && word.buff[word.offset]!='.') ||
	 (value>999 && word.buff[word.offset]!='.' &&  
	  word.buff[word.offset]!='\"' && word.buff[word.offset]!=' ' &&
	  word.buff[word.offset]!='\t' )){
	output.print_traffic_threedigits(value);
      } else {
	output.print_unsigned8(value);
      }
    } else {
      output.print_char(word.buff[word.offset++]);
    }
  }
}

int main(){
  while (getnextword()){
    processunstructuredword();
  }
}
