/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 07/23/2003 */

/* The splitter splits the input trace by time and category. The
   categories are defined by the categories.txt file that the splitter
   reads in from the current directory. Each line contains a cluster
   and a category name (alphabetic character followed by _s and
   alphanumerics up to 19 characters long). Each packet is mapped to
   the category of the first cluster it matches. There can be more
   than one cluster mapping to the same category. The first line where
   a category appears defines the color associated with it and a
   longer textual description. One separate trace file (without the
   timestamps) is created for each category. After each minute of the
   trace, the split traces are closed, we execute an external program
   that is supposed to process them and we start anew. */

#include <stdio.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "../datatypes/ip/ip.h"
#include "../datatypes/port/port.h"
#include "../datatypes/protocol/proto.h"
#include "../datatypes/cluster.h"
#include "../datatypes/counter64.h"
#include "../datatypes/timestamp.h"
#include "../readwrite/flowexportreader.h"
#include "../readwrite/assume.h"

ip srcip("Source IP");
ip destip("Dest. IP");
port srcport("SrcP");
port destport("DestP");
proto protocol("Pro");
counter64 octets("Bytes");
counter64 packets("Packets");
timestamp first("First");
timestamp last("Last");
compound flowIDrec(&srcip,&destip,&protocol,&srcport,&destport,&octets,&packets);

time_t t;

int main(int argc, char* argv[]){
  char times[100]="";
  flowexportreader f("-");
  byte* data;
  buffer b;
  b.offset=0;
  b.print_string("#First packet                       Last packet               ");
  flowIDrec.prettyprintname(b);
  b.print_char('\0');
  printf("%s\n",b.buff);
  while(data=f.getnextrecord()){
    t=(*(uint8*)data)/1000000000;
    strftime(times,100,"%Y %b %d %H:%M:%S",localtime((time_t*)&t));
    printf("%s.%09lld ",times,(*(uint8*)data)%1000000000);
    t=(*(uint8*)(data+8))/1000000000;
    strftime(times,100,"%Y %b %d %H:%M:%S",localtime((time_t*)&t));
    printf("%s.%09lld ",times,(*(uint8*)(data+8))%1000000000);
    b.offset=0;
    flowIDrec.prettyprint(b,data+16);
    b.print_char('\0');
    printf("%s\n",b.buff);
  }
}
