/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 07/15/2002 */

#include <string.h>
#include <stdlib.h>
#include "flowexportreader.h"
#include "assume.h"
#include "../datatypes/timestamp.h"
#include "../datatypes/ip/ip.h"
#include "../datatypes/port/port.h"
#include "../datatypes/protocol/proto.h"
#include "../datatypes/counter64.h"
#include "../datatypes/compound.h"

static timestamp first("First");
static timestamp last("Last");
static ip srcIP("SrcIP");
static ip dstIP("DestIP");
static proto pr("Pro");
static port srcPort("SrcPort");
static port dstPort("DestPort");
static counter64 octets("Bytes");
static counter64 packets("Packets");
static int lastoffset=first.getbinsize();
static int srcIPoffset=lastoffset+last.getbinsize();
static int dstIPoffset=srcIPoffset+srcIP.getbinsize();
static int protooffset=dstIPoffset+dstIP.getbinsize();
static int srcPortoffset=protooffset+pr.getbinsize();
static int dstPortoffset=srcPortoffset+srcPort.getbinsize();
static int octetsoffset=dstPortoffset+dstPort.getbinsize();
static int packetsoffset=octetsoffset+octets.getbinsize();
static int totalsize=packetsoffset+packets.getbinsize();

flowexportreader::flowexportreader(const char* fname, int die)
  :f(fname),dieonerror(die){
  databuffer=(byte*)malloc(totalsize);
  assumenotnull(databuffer);
}

static uint8 number;
static uint8 sysboottime;
static char is[]="Invalid syntax";
static byte scratch[8];

#define parsestuff(parsefun) if(dieonerror){f.succeedordie(parsefun);} \
  else {if(!(parsefun)){f.survivebuffererror(); return getnextrecord();}}

byte* flowexportreader::getnextrecord(){
  if(!f.getnextline()){
    return NULL;
  }
  parsestuff(f.b.parse_unsigned8(&number));
  sysboottime=1000000000*number;
  parsestuff(f.b.parse_char(',',is));
  parsestuff(f.b.parse_unsigned8(&number));
  sysboottime+=number;
  parsestuff(f.b.parse_char(',',is));
  parsestuff(f.b.parse_unsigned8(&number));
  sysboottime-=number*1000000;
  parsestuff(f.b.parse_char(',',is));
  f.b.skip_if_present('"'); /* Junk */
  parsestuff(srcIP.parse(f.b,scratch)); /* Junk */
  f.b.skip_if_present('"'); /* Junk */
  parsestuff(f.b.parse_char(',',is));
  parsestuff(packets.parse(f.b,databuffer+packetsoffset));
  parsestuff(f.b.parse_char(',',is));
  parsestuff(packets.parse(f.b,databuffer+octetsoffset));
  parsestuff(f.b.parse_char(',',is));
  parsestuff(f.b.parse_unsigned8(&number));
  *((uint8*)databuffer)=sysboottime+number*1000000;
  parsestuff(f.b.parse_char(',',is));
  parsestuff(f.b.parse_unsigned8(&number));
  *((uint8*)(databuffer+lastoffset))=sysboottime+number*1000000;
  parsestuff(f.b.parse_char(',',is)); 
  parsestuff(f.b.parse_unsigned8(&number)); /* Junk */
  parsestuff(f.b.parse_char(',',is)); 
  parsestuff(f.b.parse_unsigned8(&number)); /* Junk */
  parsestuff(f.b.parse_char(',',is));
  f.b.skip_if_present('"');
  parsestuff(srcIP.parse(f.b,databuffer+srcIPoffset));
  f.b.skip_if_present('"');
  parsestuff(f.b.parse_char(',',is));
  f.b.skip_if_present('"');
  parsestuff(dstIP.parse(f.b,databuffer+dstIPoffset));
  f.b.skip_if_present('"');
  parsestuff(f.b.parse_char(',',is));
  f.b.skip_if_present('"'); /* Junk */
  parsestuff(srcIP.parse(f.b,scratch)); /* Junk */
  f.b.skip_if_present('"'); /* Junk */
  parsestuff(f.b.parse_char(',',is)); 
  parsestuff(f.b.parse_unsigned8(&number)); /* Junk */
  parsestuff(f.b.parse_char(',',is)); 
  parsestuff(f.b.parse_unsigned8(&number)); /* Junk */
  parsestuff(f.b.parse_char(',',is));
  parsestuff(srcPort.parse(f.b,databuffer+srcPortoffset));
  parsestuff(f.b.parse_char(',',is));
  parsestuff(dstPort.parse(f.b,databuffer+dstPortoffset));
  parsestuff(f.b.parse_char(',',is));
  parsestuff(pr.parse(f.b,databuffer+protooffset));
  return databuffer;
}

flowexportreader::~flowexportreader(){
  free(databuffer);
}
