/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 06/27/2003 */

#include "populate.h"
#include "asciifile.h"
#include "../datatypes/timestamp.h"
#include "../datatypes/counter64.h"
#include "assume.h"

void read_plain(const char* filename, compound& id, hash_map& summary,
		int aggregation_type, unsigned offset){
  byte* crtid;
  unsigned nextoffset;
  if(aggregation_type==AGGREGATION_SUM_AND_COUNT){
    assume(summary.valuesize()>=16+offset,"I almost shot myself in the foot\n");
    nextoffset=offset+8;
  } else {
    assume(summary.valuesize()>=8+offset,"I almost shot myself in the foot\n");
  }
  if(aggregation_type!=AGGREGATION_SUM &&
     aggregation_type!=AGGREGATION_SUM_AND_COUNT){
    asciifilereader f(filename,id);
    if(aggregation_type==AGGREGATION_CREATE){
      while(crtid=f.getnextrecord()){
	*((uint8*)(summary[crtid]+offset))=1;
      }
    } else {
      while(crtid=f.getnextrecord()){
	(*((uint8*)(summary[crtid]+offset)))++;
      }
    }
  } else {
    unsigned idlen=id.getbinsize();
    counter64 c("Size");
    compound linestructure(&id,&c);
    asciifilereader f(filename,linestructure);
    if(aggregation_type==AGGREGATION_SUM_AND_COUNT){
      while(crtid=f.getnextrecord()){
	*((uint8*)(summary[crtid]+offset))+=*(uint8*)(crtid+idlen);
	(*((uint8*)(summary[crtid]+nextoffset)))++;
      }
    } else {
      while(crtid=f.getnextrecord()){
	*((uint8*)(summary[crtid]+offset))+=*(uint8*)(crtid+idlen);
      }
    }
  }
}

void read_timestamped(const char* filename, compound& id, hash_map& summary,
		      int aggregation_type, unsigned offset){
  byte* crtdata;
  counter64 c("Size");
  timestamp ts("Timestamp");
  compound linestructure(&ts,&id,(aggregation_type==AGGREGATION_SUM||aggregation_type==AGGREGATION_SUM_AND_COUNT?&c:NULL));
  asciifilereader f(filename,linestructure);
  unsigned nextoffset=offset+8;
  unsigned idoffset=ts.getbinsize();
  unsigned coffset=idoffset+id.getbinsize();
  if(aggregation_type==AGGREGATION_SUM_AND_COUNT){
    assume(summary.valuesize()>=16+offset,"I almost shot myself in the foot\n");
  } else {
    assume(summary.valuesize()>=8+offset,"I almost shot myself in the foot\n");
  }
  while(crtdata=f.getnextrecord()){
    switch(aggregation_type){
    case AGGREGATION_SUM:
      *((uint8*)(summary[crtdata+idoffset]+offset))+=
	*(uint8*)(crtdata+coffset);
      break;
    case AGGREGATION_COUNT:
      (*((uint8*)(summary[crtdata+idoffset]+offset)))++;
      break;
    case AGGREGATION_SUM_AND_COUNT:
      *((uint8*)(summary[crtdata+idoffset]+offset))+=
	*(uint8*)(crtdata+coffset);
      (*((uint8*)(summary[crtdata+idoffset]+nextoffset)))++;
      break;
    default:
      *((uint8*)(summary[crtdata+idoffset]+offset))=1;
    }
  }
}

void read_flow_records(const char* filename, compound& id, hash_map& summary,
		       int aggregation_type, unsigned offset){
  byte* crtid;
  unsigned nextoffset;
  if(aggregation_type==AGGREGATION_SUM_AND_COUNT){
    assume(summary.valuesize()>=16+offset,"I almost shot myself in the foot\n");
    nextoffset=offset+8;
  } else {
    assume(summary.valuesize()>=8+offset,"I almost shot myself in the foot\n");
  }
  if(aggregation_type==AGGREGATION_CREATE){
    asciifilereader f(filename,id);
    while(crtid=f.getnextrecord()){
      *((uint8*)(summary[crtid]+offset))=1;
    }
    return;
  }
  unsigned idlen=id.getbinsize();
  counter64 bytes("Bytes");
  counter64 pkts("Packets");
  unsigned pktpos=idlen+8;
  compound linestructure(&id,&bytes,&pkts);
  asciifilereader f(filename,linestructure);
  if(aggregation_type==AGGREGATION_SUM_AND_COUNT){
    while(crtid=f.getnextrecord()){
      *((uint8*)(summary[crtid]+offset))+=*(uint8*)(crtid+idlen);
      *((uint8*)(summary[crtid]+nextoffset))+=*(uint8*)(crtid+pktpos);
    }
    return;
  }
  if(aggregation_type==AGGREGATION_COUNT){
    while(crtid=f.getnextrecord()){
      *((uint8*)(summary[crtid]+offset))+=*(uint8*)(crtid+pktpos);
    }
  } else {
    while(crtid=f.getnextrecord()){
      *((uint8*)(summary[crtid]+offset))+=*(uint8*)(crtid+idlen);
    }
  }
}
