/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 06/27/2003 */

/* These helper functions populate hash tables with data they read
   from input files in ascii format. Each id gets an entry in the
   summary hash_map. */

#ifndef POPULATE_H
#define POPULATE_H

#include "../hash_map/hash_map.h"
#include "../datatypes/compound.h"
#include "../datatypes/typedefs.h"

void read_plain(const char* filename, compound& id, hash_map& summary,
		int aggregation_type, unsigned offset=0);
/* Depending on the aggregation_type, this function puts different
   things in the values associated with each id. It can expect a size
   counter (e.g. packet size) on each line and keep the sum of those
   counters for each ID, or it can just simply count the number of
   occurences of the ID, or it can just create an entry for each
   unique ID with a counter set to 1. The offset field indicates at
   what offset in the value the 8 byte counters are. */
#define AGGREGATION_SUM            0
#define AGGREGATION_COUNT          1
#define AGGREGATION_CREATE         2
#define AGGREGATION_SUM_AND_COUNT  3


void read_timestamped(const char* filename, compound& id, hash_map& summary,
		      int aggregation_type, unsigned offset=0);
/* IDs should be preceded by timestamps (which are ignored). */

void read_flow_records(const char* filename, compound& id, hash_map& summary,
		       int aggregation_type, unsigned offset=0);
/* Flow records contain flow ID, byte count and packet
   count. Mulltiple records with the same key are combined
   together. */

#endif /* POPULATE_H */
