#See AutoFocus/doc/COPYING for copyright/licensing information.
$syn_mem=($max_megs_of_memory_used>240 ? $max_megs_of_memory_used/3 :
	  $max_megs_of_memory_used-160);

if($syn_mem<10){
  die "AutoFocus needs at least 170 megabytes of memory\n";
}

$max_syn_size=int(40000*$syn_mem);

$binsize{oneminute}=60;
$binsize{tenminutes}=600;
$binsize{onehour}=3600;

$graphoptions{oneminute}="";
$graphoptions{tenminutes}="";
$graphoptions{onehour}=' -x HOUR:12:HOUR:24:HOUR:24:86400:%a ';

$label{bytes}="bits/second";
$label{packets}="packets/second";
$label{flows}="flows/minute";

$factor{bytes}=8*$sampling;
$factor{packets}=1*$sampling;
$factor{flows}=60;

@cats=();
@rules=();
open(C,"categories.txt") or die "Cannot open categories.txt\n";
while(<C>){
  my(@fields)=split(/\s+/,$_);
  if($#fields>6){
    my($category)=$fields[5];
    if($category !~ /^[a-zA-Z]\w{0,18}$/){
      die "Invalid category name: $category\n";
    }
    if($category eq "totaltraffic"){
      die "\"totaltraffic\" is not a valid category name\n";
    }
    if(defined $text{$category}){
      die "Multiple definitions for category $category\n";
    }
    push @cats,$category unless ($category eq "discard");
    $color{$category}=$fields[6];
    push @rules,[$fields[0],$fields[1],$fields[2],$fields[3],$fields[4],$color{$category}];
    splice @fields,0,7;
    $text{$category}=join(" ",@fields);
  } else {
    push @rules,[$fields[0],$fields[1],$fields[2],$fields[3],$fields[4],$color{$fields[5]}];
  }
}
close(C);

sub checkdir($){
  my($directory)=@_;
  if(! -e $directory){
    mkdir $directory,0755;
  }
  if(! -d $directory or ! -w $directory){
    die "$directory should be a writable directory\n";
  }
}

# Converts a time experessed as the number of seconds since 1970 into
# the week directory, day subdirectory, three-hour subsubdirectory,
# half hour subsubsubdirectory and minute filename.

sub get_dirs_file_intervalstarts($){
  my($rawtime)=(@_);
  my(@c)=localtime($rawtime);
  my($daystart)=$rawtime-$c[0]-60*$c[1]-3600*$c[2];
  my($weekstart)=$daystart-24*3600*$c[6];
  my($weekend)=$weekstart+7*24*3600;
  my(@s)=localtime($weekstart+12*3600);
  my(@e)=localtime($weekend-12*3600);
  my($wdir)=sprintf("%d_%02d_%02d-%02d",$s[5]+1900,$s[4]+1,$s[3],$e[3]);
  my($ddir)=sprintf("%d",$c[6]);
  my($thdir)=sprintf("%02d-%02d",$c[2]-$c[2]%3,$c[2]-$c[2]%3+3);
  my($minutes)=$c[1]+60*($c[2]%3);
  my($halfhours)=int($minutes/30+0.0001);
  my($hhdir)=sprintf("%d:%02d-%d:%02d",int($halfhours/2),30*($halfhours%2),
		     int(($halfhours+1)/2),30*(($halfhours+1)%2));
  my($minfile)=sprintf("%02d",$minutes-30*$halfhours);
  my($halfhourstart)=$rawtime-$c[0]-60*$c[1]+1800*int($c[1]/30);
  my($threehourstart)=$daystart+3600*3*int($c[2]/3);;
  return($wdir,$ddir,$thdir,$hhdir,$minfile,
	 $weekstart,$daystart,$threehourstart,$halfhourstart);
}

# Converts a time experessed as the number of seconds since 1970 into
# ten minute bin start and one hour bin start.

sub get_binstarts($){
  my($rawtime)=(@_);
  my(@c)=localtime($rawtime);
  my($tenminutestart)=$rawtime-$c[0]-60*($c[1]%10);
  my($hourstart)=$rawtime-$c[0]-60*$c[1];
  return($tenminutestart,$hourstart);
}

sub data_files($$$){
  my($endtime,$numfiles,$cat)=@_;
  my($result,$ext,$firsttimestamp);
  if($input_type eq "trace"){
    $ext=".ntt";
  } else {
    $ext=".frt";
  }
  $firsttimestamp=$endtime-60;
  if(-r "split_raw_data/$firsttimestamp/$cat$ext"){
    $result="split_raw_data/$firsttimestamp/$cat$ext";
  } else {
    $result="";
  }
  for($crt=$firsttimestamp-60;$crt>$firsttimestamp-60*$numfiles;$crt-=60){
    if(-r "split_raw_data/$crt/$cat$ext"){
      $result.=" split_raw_data/$crt/$cat$ext";
    }
  }
  return "/dev/null $result";
}

sub system_and_test($){
  my($command)=@_;
  system($command);
  if($?){
    my($exit_value,$signal_num)=($? >> 8 , $? & 127);
    die("$command\n failed with exit code $exit_value, signal $signal_num and \$\? $?\n");
  }
}

sub system_test_and_compress($$){
  my($command,$outputfile)=@_;
  system("$command > tempoutput.txt");
  if($?){
    my($exit_value,$signal_num)=($? >> 8 , $? & 127);
    die("$command | gzip > $outputfile\n failed with exit code $exit_value, signal $signal_num and \$\? $?\n");
  }
  system("rm -f tempoutput.txt.gz; gzip tempoutput.txt");
  system("mv tempoutput.txt.gz $outputfile");
}

sub update_rrd($$){
  my($endtime,$binsz)=@_;
  my(%dataline);
  my($type,$cat);
  foreach $type ("bytes","packets","flows"){
    $dataline{$type}=sprintf("%d",$endtime-$binsize{$binsz}/2);
  }
  foreach $cat (@cats){
    my ($datafiles)=data_files($endtime,$binsize{binsz}/60,$cat);
    my (@count);
    if ($datafiles){
      if($input_type eq "trace"){
	@count=split(/\s+/,`cat $datafiles | $bin_dir/counteverything`);
      } else {
	@count=split(/\s+/,`cat $datafiles | $bin_dir/counteverythingnf`);
      }
    } else {
      @count=("0","bytes","0","packets","0","flows");
    }
    $dataline{bytes}.=":".($count[0]*$factor{bytes}/$binsize{$binsz});
    $dataline{packets}.=":".($count[2]*$factor{packets}/$binsize{$binsz});
    $dataline{flows}.=":".($count[4]*$factor{flows}/$binsize{$binsz});
  }
  foreach $type ("bytes","packets","flows"){
    system_and_test ("$rrdtool update traffic-$type-$binsz.rrd $dataline{$type}");
  }
}
sub draw_gifs($$$$$){
  my($binsz,$directory,$endtime,$intervalstart,$intervalend)=@_;
  my($type,$cat,$starttime);
  my(%commandline);
  $starttime=$endtime-200*$binsize{$binsz};
  foreach $type ("bytes","packets","flows"){
    $commandline{$type}="$rrdtool graph $directory/$type.gif -h200 --start $starttime --end $endtime --vertical-label $label{$type} $graphoptions{$binsz} VRULE:$intervalstart#000000 VRULE:$intervalend#000000 >/dev/null";
    foreach $category (@cats){
      if($category eq $cats[0]){
	$commandline{$type}.=" DEF:$category=traffic-$type-$binsz.rrd:$category:AVERAGE AREA:$category#$color{$category}";
      }else{
	$commandline{$type}.=" DEF:$category=traffic-$type-$binsz.rrd:$category:AVERAGE STACK:$category#$color{$category}";
      }
      checkdir("$directory/$category");
      system_and_test("$rrdtool graph $directory/$category/$type.gif -h100 --start $starttime --end $endtime --vertical-label $label{$type} DEF:$category=traffic-$type-$binsz.rrd:$category:AVERAGE AREA:$category#$color{$category} VRULE:$intervalstart#000000 VRULE:$intervalend#000000 >/dev/null");
    }
    system_and_test($commandline{$type});
  }
}

sub write_categories_js($){
  my ($dir)=@_;
  open(CATJS,">$dir/categories.js") or die "Cannot write $dir/categories.js\n";
  print CATJS "var categories=[".join(",\n",map("[\"$color{$_}\",\"$_\",\"$text{$_}\"]",@cats))."];\n";
  print CATJS "var cat_details=[".join(",\n",map("[".join(",",map("\"$_\"",@$_))."]",@rules))."];\n";
  close(CATJS);
}

sub build_half_hour_synopses($$){
  my ($endtime,$cat)=@_;
  my ($datafiles)=data_files($endtime,30,$cat);
  if($input_type eq "trace"){
    system_test_and_compress("cat $datafiles | $bin_dir/summarizer --trace - --maxs $max_syn_size","summaries/halfhour/$cat/bytes.csyn.gz");
    system_test_and_compress("cat $datafiles | $bin_dir/summarizer --trace - --countpackets --maxs $max_syn_size","summaries/halfhour/$cat/packets.csyn.gz");
    system_test_and_compress("cat $datafiles | $bin_dir/summarizer --trace - --countflows --maxs $max_syn_size --seed $seed","summaries/halfhour/$cat/flows.fsyn.gz");
  } else {
    system_test_and_compress("cat $datafiles | $bin_dir/summarizer --flowrecfile - --maxs $max_syn_size","summaries/halfhour/$cat/bytes.csyn.gz");
    system_test_and_compress("cat $datafiles | $bin_dir/summarizer --flowrecfile - --countpackets --maxs $max_syn_size","summaries/halfhour/$cat/packets.csyn.gz");
    system_test_and_compress("cat $datafiles | $bin_dir/summarizer --flowrecfile - --countflows --maxs $max_syn_size --seed $seed","summaries/halfhour/$cat/flows.fsyn.gz");
  }
}

sub build_js_files($$){
  my ($syndir,$webdir)=@_;
  system_test_and_compress("$bin_dir/autofocus --syn $syndir/bytes.csyn.gz --res 50 -q m --sam $sampling","$webdir/bytes.clr.gz");
  system_test_and_compress("$bin_dir/autofocus --syn $syndir/packets.csyn.gz --res 50 -q m --sam $sampling --countp","$webdir/packets.clr.gz");
  system_test_and_compress("$bin_dir/autofocus --syn $syndir/flows.fsyn.gz --res 50 -q m --sam $sampling --countf","$webdir/flows.clr.gz");
  foreach $type ("bytes","packets","flows"){
    system_and_test("$bin_dir/autofocus --cl $webdir/$type.clr.gz --res 50 --ja | $bin_dir/humanizer > $webdir/$type"."_2.js");
    system_and_test("$bin_dir/autofocus --cl $webdir/$type.clr.gz --res 20 --ja | $bin_dir/humanizer > $webdir/$type"."_5.js");
    system_and_test("$bin_dir/autofocus --cl $webdir/$type.clr.gz --res 10 --ja | $bin_dir/humanizer > $webdir/$type"."_10.js");
    system_and_test("$bin_dir/autofocus --cl $webdir/$type.clr.gz --res 5 --ja  | $bin_dir/humanizer > $webdir/$type"."_20.js");
  }
}

sub write_navbar_js($$$){
  my($dir,$rawtime,$ilength)=@_;
  my(@months)=("January","February","March","April","May","June","July","August","September","October","November","December");
  my(@days)=("Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday");
  my ($title,$firstrow,$srowleft,$srowcenter,$srowright);
  if($ilength eq "oneweek"){
    my(@crt)=localtime($rawtime);
    my(@prev)=localtime($rawtime-6*24*3600-2*3600);
    $title="week $months[$prev[4]] $prev[3] ".($prev[5]+1900)." - $months[$crt[4]] $crt[3] ".($crt[5]+1900);
    $firstrow="\"\",\"Current Week\"";
    my($pwdir)=get_dirs_file_intervalstarts($rawtime-7*24*3600-2*3600);
    $srowleft="\"../$pwdir\",\"Previous week\"";
    $srowcenter="[\"\",\"Days of current week\"],[0,\"Sunday\"],[1,\"Monday\"],[2,\"Tuesday\"],[3,\"Wednesday\"],[4,\"Thursday\"],[5,\"Friday\"],[6,\"Saturday\"]";
    ($pwdir)=get_dirs_file_intervalstarts($rawtime+7*24*3600-2*3600);
    $srowright="\"../$pwdir\",\"Next week\"";
  } else {
    if($ilength eq "oneday"){
      my(@c)=localtime($rawtime);
      $title="$days[$c[6]] $months[$c[4]] $c[3] ".($c[5]+1900);
      $firstrow="\"..\",\"Current Week\"";
      my($pwdir,$pddir)=get_dirs_file_intervalstarts($rawtime-24*3600-2*3600);
      $srowleft="\"../../$pwdir/$pddir\",\"Previous day\"";
      $srowcenter="[\"\",\"3 hour intervals\"],[\"00-03\",\"0-3\"],[\"03-06\",\"3-6\"],[\"06-09\",\"6-9\"],[\"09-12\",\"9-12\"],[\"12-15\",\"12-15\"],[\"15-18\",\"15-18\"],[\"18-21\",\"18-21\"],[\"21-24\",\"21-24\"]";
      my($pwdir,$pddir)=get_dirs_file_intervalstarts($rawtime+24*3600-2*3600);
      $srowright="\"../../$pwdir/$pddir\",\"Next day\"";
    } else {
      if($ilength eq "threehours"){
	my(@c)=localtime($rawtime);
	$title="$days[$c[6]] $months[$c[4]] $c[3] ".($c[5]+1900)." ".($c[2]-2).":00-".($c[2]+1).":00";
	$firstrow="\"..\",\"Current day\"";
	my($pwdir,$pddir,$pthdir)=get_dirs_file_intervalstarts($rawtime-3*3600-2*1800);
	$srowleft="\"../../../$pwdir/$pddir/$pthdir\",\"Previous 3 hours\"";
	my(@h)=($c[2]-2,$c[2]-1,$c[2],$c[2]+1);
	$srowcenter="[\"\",\"half hour intervals\"],".join(",",map(
          "[\"$_:00-$_:30\",\"$h[$_]:00-$h[$_]:30\"],[\"$_:30-".($_+1).":00\",\"$h[$_]:30-".($h[$_+1]).":00\"]",(0,1,2)));
	my($pwdir,$pddir,$pthdir)=get_dirs_file_intervalstarts($rawtime+3*3600-2*1800);
	$srowright="\"../../../$pwdir/$pddir/$pthdir\",\"Next 3 hours\"";
      } else {
	my(@c)=localtime($rawtime);
	my($halfhours)=2*$c[2]+int($c[1]/30);
	$title="$days[$c[6]] $months[$c[4]] $c[3] ".($c[5]+1900)." ".
	  sprintf("%d:%02d-%d:%02d",int($halfhours/2),30*($halfhours%2),
		     int(($halfhours+1)/2),30*(($halfhours+1)%2));
	$firstrow="\"..\",\"3 hour interval\"";
	my($pwdir,$pddir,$pthdir,$phhdir)=get_dirs_file_intervalstarts($rawtime-45*60);
	$srowleft="\"../../../../$pwdir/$pddir/$pthdir/$phhdir\",\"Previous half hour\"";
	$srowcenter="[\"\",\"\"]";
	my($pwdir,$pddir,$pthdir,$phhdir)=get_dirs_file_intervalstarts($rawtime+15*60);
	$srowright="\"../../../../$pwdir/$pddir/$pthdir/$phhdir\",\"Next half hour\"";
      }
    }
  }
  open(NB,">$dir/nav_bar.js") or die "Cannot write to $dir/nav_bar.js\n";
  print NB "var title=\"$title\";\n";
  print NB "var first_row=[[$firstrow]];\n";
  print NB "var second_row_left=[[$srowleft]];\n";
  print NB "var second_row_center=[$srowcenter];\n";
  print NB "var second_row_right=[[$srowright]];\n";
  close(NB);
}

sub update_one_minute_gifs($){
  my($endtime)=@_;
  my($wdir,$ddir,$thdir,$hhdir,$minfile,$weekstart,$daystart,$threehourstart,
     $halfhourstart)=get_dirs_file_intervalstarts($endtime-1);
  update_rrd($endtime,"oneminute");
  draw_gifs("oneminute","$web_dir/$wdir/$ddir/$thdir/$hhdir",
	    $endtime-1,$halfhourstart,$phalfhourstart+1800);
  draw_gifs("oneminute","$web_dir/$wdir/$ddir/$thdir",
	      $endtime-1,$threehourstart,$threehourstart+3*3600);
}

sub update_ten_minute_gifs($){
  my($endtime)=@_;
  my($wdir,$ddir,$thdir,$hhdir,$minfile,$weekstart,$daystart,$threehourstart,
     $halfhourstart)=get_dirs_file_intervalstarts($endtime-1);
    update_rrd($endtime,"tenminutes");
  draw_gifs("tenminutes","$web_dir/$wdir/$ddir/",
	    $endtime,$daystart,$daystart+24*3600);
}

sub update_one_hour_gifs($){
  my($endtime)=@_;
  my($wdir,$ddir,$thdir,$hhdir,$minfile,$weekstart,$daystart,$threehourstart,
     $halfhourstart)=get_dirs_file_intervalstarts($endtime-1);
  update_rrd($endtime,"onehour");
  draw_gifs("onehour","$web_dir/$wdir",$endtime-1,$weekstart,
	    $weekstart+7*24*3600);
}

sub combine_summaries($$){
  my($srcd,$destd)=@_;
  foreach $cat (@cats){
    my($srcdir)="$srcd/$cat";
    my($destdir)="$destd/$cat";
    if(-f "$destdir/bytes.csyn.gz"){
      system_and_test("$bin_dir/summarizer  --maxs $max_syn_size --synopses ".
	     "$srcdir/bytes.csyn.gz,$destdir/bytes.csyn.gz ".
	     ">$destdir/bytes.csyn");
      system("rm $destdir/bytes.csyn.gz ; gzip $destdir/bytes.csyn");
      system_and_test("$bin_dir/summarizer --countp --maxs $max_syn_size --synopses ".
	     "$srcdir/packets.csyn.gz,$destdir/packets.csyn.gz ".
	     ">$destdir/packets.csyn");
      system("rm $destdir/packets.csyn.gz ; gzip $destdir/packets.csyn");
      system_and_test("$bin_dir/summarizer --countf --maxsize $max_syn_size --seed $seed --synopses ".
	     "$srcdir/flows.fsyn.gz,$destdir/flows.fsyn.gz ".
	     ">$destdir/flows.fsyn");
      system("rm $destdir/flows.fsyn.gz ; gzip $destdir/flows.fsyn");
    } else {
      system("cp $srcdir/* $destdir");
    }
  }
}

return 1;
