#!/usr/bin/perl
#See AutoFocus/doc/COPYING for copyright/licensing information.

require "./AutoFocusConfig";
push @INC,$script_dir;
require AutoFocus;

if($#ARGV!=0) {die("Usage process_bin.pl end_of_bin_time_stamp\n");}

$crttime=$ARGV[0]-1;

($wdir,$ddir,$thdir,$hhdir,$minfile,$weekstart,$daystart,$threehourstart,
 $halfhourstart)=get_dirs_file_intervalstarts($crttime);
update_structure();
update_plots();
add_reports();
remove_temp_data();

sub initialize_directory($$){
  my($dir,$type)=@_;
  checkdir($dir);
  write_categories_js($dir);
  write_navbar_js($dir,$crttime,$type);
  foreach $category (@cats){
    checkdir("$dir/$category");
  }
}

sub update_structure(){
  initialize_directory("$web_dir/$wdir","oneweek");
  initialize_directory("$web_dir/$wdir/$ddir","oneday");
  initialize_directory("$web_dir/$wdir/$ddir/$thdir","threehours");
  initialize_directory("$web_dir/$wdir/$ddir/$thdir/$hhdir","halfhour");
  open(F,">$web_dir/latest_directory.js") or 
    die "Cannot write $web_dir/latest_directory.js";
  print F "var directory = \"$wdir/$ddir/$thdir/$hhdir/\";\n";
  close(F);
}

sub update_plots(){
  #Updating rrd databases and gifs
  update_one_minute_gifs($crttime+1);
  ($tenminutebin,$onehourbin)=get_binstarts($crttime);
  $lasttenminutebin=`cat tenminutebin`;
  chop($lasttenminutebin);
  if($lasttenminutebin and ($lasttenminutebin ne $tenminutebin)){
    update_ten_minute_gifs($lasttenminutebin+600);
  }
  if($minfile eq "9"){
    update_ten_minute_gifs($crttime+1);
    system("cat /dev/null > tenminutebin");
  } else {
    system("echo $tenminutebin > tenminutebin");
  }
  $lastonehourbin=`cat onehourbin`;
  chop($lastonehourbin);
  if($lastonehourbin and ($lastonehourbin ne $onehourbin)){
    update_one_hour_gifs($lastonehourbin+3600);
  }
  if($minfile eq "29" and $hhdir =~ /^(\d):30-/){
    update_one_hour_gifs($crttime);
    system("cat /dev/null > onehourbin");
  } else {
    system("echo $onehourbin > onehourbin");
  }

  #Generating the "pretty gifs"
  if($minfile eq "9"){ #Write final gif for previous half hour interval
    $previntervaltime=$crttime-1800;
    ($pwdir,$pddir,$pthdir,$phhdir,$pminfile,$pweekstart,$pdaystart,
     $pthreehourstart,$phalfhourstart)=
       get_dirs_file_intervalstarts($previntervaltime);
    if(-d "$web_dir/$pwdir/$pddir/$pthdir/$phhdir"){
      draw_gifs("oneminute","$web_dir/$pwdir/$pddir/$pthdir/$phhdir",
		$crttime,$phalfhourstart,$phalfhourstart+1800);
    }
  }
  if($minfile eq "9" and $hhdir eq "0:00-0:30"){
    #Write final gif for previous three hour interval
    $previntervaltime=$crttime-3*3600;
    ($pwdir,$pddir,$pthdir,$phhdir,$pminfile,$pweekstart,$pdaystart,
     $pthreehourstart,$phalfhourstart)=
       get_dirs_file_intervalstarts($previntervaltime);
    if(-d "$web_dir/$pwdir/$pddir/$pthdir"){
      draw_gifs("oneminute","$web_dir/$pwdir/$pddir/$pthdir",
		$crttime,$pthreehourstart,$pthreehourstart+3*3600);
    }
  }
  if($minfile eq "9" and $thdir eq "03-06" and $hhdir eq "1:30-2:00"){
      #Write final gif for previous day interval
    $previntervaltime=$crttime-24*3600;
    ($pwdir,$pddir,$pthdir,$phhdir,$pminfile,$pweekstart,$pdaystart,
     $pthreehourstart,$phalfhourstart)=
       get_dirs_file_intervalstarts($previntervaltime);
    if(-d "$web_dir/$pwdir/$pddir"){
      draw_gifs("tenminutes","$web_dir/$pwdir/$pddir",
		$crttime,$pdaystart,$pdaystart+24*3600);
    }
  }
  if($minfile eq "29" and $ddir eq "0" and $thdir eq "15-18" and 
     $hhdir eq "0:30-1:00"){
    #Write final gif for previous week interval
    $previntervaltime=$crttime-7*24*3600;
    ($pwdir,$pddir,$pthdir,$phhdir,$pminfile,$pweekstart,$pdaystart,
     $pthreehourstart,$phalfhourstart)=
       get_dirs_file_intervalstarts($previntervaltime);
    if(-d "$web_dir/$pwdir"){
      draw_gifs("onehour","$web_dir/$pwdir",$crttime,$pweekstart,
		$pweekstart+7*24*3600);
    }
  }
}

sub compute_reports($$){
  my($syndir,$webdir)=@_;
  if($syndir == "summaries/halfhour"){
    my($hhstart)=`cat  summaries/halfhour/start`;
    chop($hhstart);
    foreach $category (@cats){
      build_half_hour_synopses($hhstart+1800,$category);
    }
  }
  system_test_and_compress("$bin_dir/summarizer  --maxs $max_syn_size --synopses ".
	 (join(",",map("$syndir/$_/bytes.csyn.gz",@cats))),
	 "$syndir/bytes.csyn.gz");
  system_test_and_compress("$bin_dir/summarizer --countp --maxs $max_syn_size --synopses ".
	 (join(",",map("$syndir/$_/packets.csyn.gz",@cats))),
	 "$syndir/packets.csyn.gz");
  system_test_and_compress("$bin_dir/summarizer --countf --m $max_syn_size --seed $seed --sy ".
	 (join(",",map("$syndir/$_/flows.fsyn.gz",@cats))),
	 "$syndir/flows.fsyn.gz");
  unless ((($cpu eq "slow") and ($syndir eq "summaries/halfhour")) or
	  (($cpu eq "slow") and ($syndir eq "summaries/threehour")) or
	  (($cpu eq "medium") and ($syndir eq "summaries/halfhour"))){
    foreach $category (@cats){
      build_js_files("$syndir/$category","$webdir/$category");
    }
  }
  build_js_files($syndir,$webdir);
  if($syndir eq "summaries/halfhour"){
    combine_summaries($syndir,"summaries/threehour");
  }
  if($syndir eq "summaries/threehour"){
    combine_summaries($syndir,"summaries/day");
  }
  if($syndir eq "summaries/day"){
    combine_summaries($syndir,"summaries/week");
  }
  system("rm $syndir/*.csyn.gz $syndir/*.fsyn.gz ".
	 "$syndir/*/*.csyn.gz $syndir/*/*.fsyn.gz");
  system("cat /dev/null > $syndir/webdir");
}

sub add_reports(){
  $lasthhd=`cat summaries/halfhour/webdir`;
  chop($lasthhd);
  $hhd="$web_dir/$wdir/$ddir/$thdir/$hhdir";
  if($lasthhd and ($lasthhd ne $hhd)){
    compute_reports("summaries/halfhour",$lasthhd);
  }
  $lastthd=`cat summaries/threehour/webdir`;
  chop($lastthd);
  $thd="$web_dir/$wdir/$ddir/$thdir";
  if($lastthd and ($lastthd ne $thd)){
    compute_reports("summaries/threehour",$lastthd);
  }
  $lastdd=`cat summaries/day/webdir`;
  chop($lastdd);
  $dd="$web_dir/$wdir/$ddir";
  if($lastdd and ($lastdd ne $dd)){
    compute_reports("summaries/day",$lastdd);
  }
  $lastwd=`cat summaries/week/webdir`;
  chop($lastwd);
  $wd="$web_dir/$wdir";
  if($lastwd and ($lastwd ne $wd)){
    compute_reports("summaries/week",$lastwd);
  }
  system("echo $halfhourstart > summaries/halfhour/start");
  system("echo $hhd > summaries/halfhour/webdir");
  system("echo $thd > summaries/threehour/webdir");
  system("echo $dd > summaries/day/webdir");
  system("echo $wd > summaries/week/webdir");

  if($minfile eq "29"){
    compute_reports("summaries/halfhour",$hhd);
    if($hhdir eq "2:30-3:00"){
      compute_reports("summaries/threehour",$thd);
      if($thdir eq "21-24"){
	compute_reports("summaries/day",$dd);
	if($ddir eq "6"){
	  compute_reports("summaries/week",$wd);
	}
      }
    }
  }
}

sub remove_temp_data(){
  $clean_up_thresh=$halfhourstart-3600;
  foreach $file (glob("split_raw_data/*")){
    $file =~ /split_raw_data\/(\d+)$/;
    if($1<$clean_up_thresh){
      system("rm -rf $file");
    }
  }
}
